define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "thornberry/app/Component/_Component",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "dojo/dom-construct",
    "dojo/text!./Templates/ProductivityScoreCard.html",
    "dojo/dom-style",
    "dojo/dom-class",
    "dojo/query",
    "thornberry/serverRequest",
    "dojo/currency",
    "dojo/NodeList-dom"
],function(
    declare,
    lang,
    arrayUtils,
    _Component,
    _TemplatedMixin,
    _WidgetsInTemplateMixin,
    domConstruct,
    template,
    domStyle,
    domClass,
    query,
    serverRequest,
    currency
) {

    var detailStructure = [
        { name : "Patient"           , sizing: "small", alignment: "left" },
        { name : "Claim ID"          , sizing: "small", alignment: "left" },
        { name : "Transaction Type"  , sizing: "medium", alignment: "center" },
        { name : "Amount"            , sizing: "small", alignment: "right" },
        { name : "Activity Date"     , sizing: "medium-small", alignment: "center" }
    ];

    var Row = declare([_Component,_TemplatedMixin,_WidgetsInTemplateMixin],{
        template: 
            "<div class='content-flex-container'>"+
                "<div class='flex-field-container'>"+
                    "<div class='field-name field-name-grand'>{name}</div>"+
                    "<div class='field-value field-value-medium-large app-component-scorecard-row' style='text-align:right;padding-right:10px' data-dojo-attach-event='onclick:_clickPaymentAdjustment'>{paymentAdjustmentCount}</div>"+
                    "<div class='field-value field-value-medium-large app-component-scorecard-row' style='text-align:right;padding-right:10px' data-dojo-attach-event='onclick:_clickSentClaim'>{sentClaimCount}</div>"+
                    "<div class='field-value field-value-medium-large app-component-scorecard-row' style='text-align:right;padding-right:10px' data-dojo-attach-event='onclick:_clickClaimFollowUp'>{claimFollowUpCount}</div>"+
                "</div>"+
            "</div>",

        getInitialProps: function() {
            var t = this;
            return lang.mixin(t.inherited(arguments),{
                onClick: null,
                item: null,
                getDisplayValue: null
            });
        }, 

        postMixInProperties: function() {
            var t = this;
            t.inherited(arguments);

            t.templateString = lang.replace(t.template,t.props);

            if (t.props && t.props.item) {
                var item = t.props.item;
                for (key in item) {
                    item[key] = t.props.getDisplayValue ? t.props.getDisplayValue(key,item[key]) : item[key];
                }

                t.templateString = lang.replace(t.template,item);
            }
        },

        buildRendering: function() {
            var t = this;
            t.inherited(arguments);
            var clickable = t.get("clickable");
            query(".field-value",t.domNode).forEach(function(node) {
                domClass.toggle(node,"app-component-scorecard-row",clickable && node.innerText != "0");
            });
        },

        onClick: function(type) {
            var t = this;
            if (t.props.onClick && t.get("clickable") && t.props.item[type+"Count"] != 0) {
                t.props.onClick(type,t.props.item.id);
            }
        },

        _clickPaymentAdjustment: function() {
            this.onClick("paymentAdjustment");
        },

        _clickSentClaim: function() {
            this.onClick("sentClaim");
        },

        _clickClaimFollowUp: function() {
            this.onClick("claimFollowUp");
        },

        _getClickableAttr: function() {
            var t = this;
            return (t.props && t.props.item && t.props.item.name != " - ");
        }
    });

    var DetailRow = declare([_Component,_TemplatedMixin,_WidgetsInTemplateMixin],{
        template: 
            "<div class='content-flex-container'>"+
                "<div class='flex-field-container' title='{description}'>"+
                    "{fields}"+
                "</div>"+
            "</div>",

        getInitialProps: function() {
            var t = this;
            return lang.mixin(t.inherited(arguments),{
                onClick: null,
                item: null,
                getDisplayValue: null
            });
        }, 

        postMixInProperties: function() {
            var t = this;
            t.inherited(arguments);

            if (t.props && t.props.item) {
                var item = t.props.item;
                for (key in item) {
                    item[key] = t.props.getDisplayValue ? t.props.getDisplayValue(key,item[key]) : item[key];
                }

                var fields = "";
                arrayUtils.forEach(detailStructure,function(field) {
                    if (typeof t.props.item[field.name] !== "undefined") {
                        fields += lang.replace(
                            "<div class='field-value field-value-{sizing}' style='text-align:{alignment};'>{value}</div>",
                            lang.mixin(field,{ value: item[field.name] })
                        );
                    }
                });

                t.templateString = lang.replace(t.template,{ fields: fields, description: [item.Description] });
            }
        }
    });

    return declare([_Component,_TemplatedMixin,_WidgetsInTemplateMixin],{
        templateString: template,

            getInitialProps: function() {
                var t = this;
                return lang.mixin(t.inherited(arguments),{
                    data: null,
                    detail: false,
                    loading: false
                });
            }, 

            refreshRendering: function() {
                var t = this;
                domConstruct.empty(t.itemContainer);
                domConstruct.empty(t.detailItemContainer);
                domConstruct.empty(t.detailItemHeader);

                if (t.props) {
                    domStyle.set(t.mainView,"display",(t.props.detail ? "none" : ""));
                    domStyle.set(t.detailView,"display",(t.props.detail ? "" : "none"));
                    domStyle.set(t.loadingNode,"display",(t.props.loading ? "" : "none"));

                    if (!t.props.detail && t.props.data) {
                        t.props.detailData = null;
                        arrayUtils.forEach(t.props.data,function(item) {
                            t.itemContainer.appendChild(new Row({
                                props: {
                                    item: item,
                                    getDisplayValue: t.props.getDisplayValue,
                                    onClick: lang.hitch(t,"onClickCell")
                                }
                            }).domNode);
                        });
                    } else if (t.props.detail && t.props.detailData) {
                        t.updateDetailItemHeader();
                        arrayUtils.forEach(t.props.detailData,function(item) {
                            t.detailItemContainer.appendChild(new DetailRow({
                                props: {
                                    item: item,
                                    getDisplayValue: lang.hitch(t,"getDetailDisplayValue"),
                                }
                            }).domNode);
                        });
                    }
                }
            },

            updateDetailItemHeader: function() {
                var t = this;
                if (t.props && t.props.detailData && t.props.detailData.length) {
                    var item = t.props.detailData[0];
                    arrayUtils.forEach(detailStructure,function(field) {
                        if (typeof item[field.name] !== "undefined") {
                            var headerField = lang.replace("<div class='field-value field-value-{sizing}' style='text-align:{alignment}'>{name}</div>",field);
                            var headerNode = domConstruct.toDom(headerField);
                            t.detailItemHeader.appendChild(headerNode);
                        }
                    });
                }
            },

            getDetailDisplayValue: function(key,value) {
                if (key == "Amount") {
					value = (value === "" ? "$ - " : "$"+currency.format(value));
				}
                return value;
            },

            onClickCell: function(type,id) {
                var t = this;
                var item = arrayUtils.filter(t.props.data,function(item) { return item.id == id; })[0];
                var typeDisplay = {
                    "paymentAdjustment" : "Payments/Adjustments  for ",
                    "sentClaim" : "Claims Sent by ",
                    "claimFollowUp" : "Claims Followed-up on by "
                }[type];

                t.detailHeader.innerText = typeDisplay + item.name;
                t.set("props",{ detail: true, loading: true });
                
                serverRequest("Billing.AccountsReceivable.DashboardService","GetProductivityDetails",{ 
                    "type" : type, 
                    "userId" : item.userId 
                }).then(function(results) {
                    t.set("props",{ detailData: results, loading: false });
                },function() {
                    t.set("props",{ loading: false });
                });
            },

            _clickReturn: function() {
                this.set("props",{ detail: false });
            }
    });
});