define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "thornberry/app/Component/_Component",
    "thornberry/app/Component/_EditorViewMixin",
    "./CloseFinancialPeriodEditor",
    "thornberry/app/service/BillingManagement",
    "dojo/Deferred",
    "thornberry/app/utils",
    "dojo/when",
    "thornberry/app/Service/Settings",
    "dojo/date",
    "thornberry/ReportManagerII",
    "dojo/dom-construct",
    "./FinancialPeriodSummaryView",
    "dijit/Dialog",
	"dijit/form/Button"
],function(
    declare,
    lang,
    arrayUtils,
    _Component,
    _EditorViewMixin,
    CloseFinancialPeriodEditor,
    service,
    Deferred,
    utils,
    when,
    settingsService,
    date,
    ReportManager,
    domConstruct,
    FinancialPeriodSummaryView,
    Dialog,
    Button
) {
    return declare([_Component,_EditorViewMixin], {
        title: "Close Financial Period",

        EditorClass: CloseFinancialPeriodEditor,

        lockCheckEnabled: true,
        objectType: "Billing.FinancialPeriod",

        objectId: null,
        financialPeriod: null,

        settings: null,
        financialReports: null,

        getTitle: function() {
            return this.title;
        },

        getInitialProps: function() {
            return lang.mixin(this.inherited(arguments),{
                mode: "edit", // view or edit
                financialPeriods: null,
                onSave: null
            });
        },

        getPageProps: function() {
            var t = this;
            var props = lang.mixin(t.inherited(arguments),{
                actions: [
                    { label: "Cancel", onClick: function() { t.set("props",{ objectId: null }); t.props.onCloseView(); } },
                    { label: "Close Period", onClick: lang.hitch(t,"close") }
                ],
                printButtonShown: true,
                accountSummaryShown: false,

                onClickPrint: lang.hitch(t.reportManager,"show")
            });
            return props;
        },

        buildRendering: function() {
            var t = this;
            t.inherited(arguments);

            t.reportManager = new ReportManager();
            t.reportManager.startup();
            t.reportManager.set("report",null);
        },

        setDefaultEditorProps: function(editor) {
            var t = this;
            t.inherited(arguments);
            t.editor.set("props",{
                setParent: lang.hitch(t,"set")
            });
        },

        onLockFailure: function(deferred) {
            // default implementation - if we were in view mode before switching to edit, then stay in view mode, otherwise pop the view
            var t = this;
            t.props.mode = "edit";
            t.props.objectId = null;
            t.editor.set("props",{
                value: null,
                disabled: false 
            });
            deferred.resolve();
        },

        onLockComplete: function(deferred) {
            var t = this;
            t.inherited(arguments);
            if (deferred) {
                t.set("objectId",t.props.objectId);
            }
        },

        refreshRendering: function() {
            var t = this;
            if (t.isShown) {
                if (!t.settings) {
                    t.addReadyPromise(settingsService.get("billingComponent").then(function(settings) { 
                        t.settings = settings; 
                        t.editor.set("props",{ settings: settings });
                    }));
                }
                if (!t.financialReports) {
                    t.addReadyPromise(service.getFinancialReports().then(function(financialReports) {
                        t.financialReports = financialReports;
                        t.editor.set("props",{ financialReports: financialReports });
                    }));
                }
                t.editor.set("props",{ 
                    financialPeriods: t.props.financialPeriods, 
                    onPushView: t.props.onPushView,
                    onGenerateReport: function(generatedItem) {
                        domConstruct.place(generatedItem.domNode,t.reportManager.generatedItemContainer,"first");
                        t.reportManager.show();
                    }
                });
            }
            if (!t.props.objectId) {
                t.set("objectId","");
            }
            t.inherited(arguments);
        },

        _setObjectIdAttr: function(value) {
            var t = this;
            if (value && t.objectId !== value) {
                t.addReadyPromise(service.getNextFinancialPeriod(value).then(function(financialPeriod) {
                    if (t.objectId === value) {
                        var nextCloseDate = utils.conversions.date(null,financialPeriod.date,"set");
                        if (date.compare(new Date(),nextCloseDate,"date") > 0) {
                            t.set("financialPeriod",financialPeriod);
                        } else {
                            utils.showMessage("This financial period will end on "+utils.formatters.date("date")(financialPeriod)+" and cannot yet be closed.");
                            t.set("objectId",null);
                            t.unlock(value);
                            value = null;
                        }
                    }
                }));
            } else if (!value) {
                t.set("financialPeriod",null);
            }
            t.objectId = value;
        },

        _setFinancialPeriodAttr: function(value) {
            var t = this;
            t.financialPeriod = value;
            t.editor.set("props",{
                value: value
            });
        },

        startSave: function() {
            var t = this;
            var deferred = new Deferred();
            if (t.editor.reportItems.length && arrayUtils.some(t.editor.reportItems,function(item) {
                return !item.reportNameField.validate();
            })) {
                utils.showMessage("Please correct any invalid fields.");
                deferred.reject();
            } else {
                t.inherited(arguments).then(deferred.resolve,deferred.reject);
            }
            return deferred.promise;
        },

        save: function() {
            var t = this;
            var deferred = new Deferred();
            t.confirm().then(function() {
                var data = t.editor.getSaveData();
                var promise = service.saveFinancialPeriod(data);
                promise.then(function(financialPeriodId) {
                    t.savedFinancialPeriodId = financialPeriodId;
                    if (t.props.onSave) {
                        t.props.onSave();
                    }
                    t.defer(function() {
                        t.set("props",{ "objectId": null });
                    });
                });
                t.props.onSetLoadingPromise(promise,"Processing...");
                promise.then(deferred.resolve,deferred.reject);
            },deferred.reject);
            return deferred.promise;
        },

        close: function() {
            var t = this;
            t.startSave().then(function() {
                var view = new FinancialPeriodSummaryView({
                    props: {
                        objectId: t.savedFinancialPeriodId
                    }
                });
                t.props.onAddView(view,true);
            });
        },

        confirm: function() {
            var t = this;
            var deferred = new Deferred();
            var dateRange = utils.formatters.date("startDate")(t.financialPeriod)+" to "+utils.formatters.date("date")(t.financialPeriod);
            var messageDialog = new Dialog({
                title: "Verify Closing Financial Period",
                content: 
                    '<div  class="content-area">'+
                        '<div class="content-area-title">'+
                            '<div style="display:inline-block" class="content-area-title-text">Summary</div>'+
                        '</div>'+
                        '<div class="content-flex-container">'+
                            '<div class="flex-field-container">'+
                                '<div class="field-name field-name-large">'+
                                    '<label>Location Type</label>'+
                                '</div>'+
                                '<div class="field-value">'+
                                    '<label>'+t.financialPeriod.locationTypeDisplay+'</label>'+
                                '</div>'+
                            '</div>'+
                        '</div>'+
                        '<div class="content-flex-container">'+
                            '<div class="flex-field-container">'+
                                '<div class="field-name field-name-large">'+
                                    '<label>Financial Period</label>'+
                                '</div>'+
                                '<div class="field-value">'+
                                    '<label>'+dateRange+'</label>'+
                                '</div>'+
                            '</div>'+
                        '</div>'+
                    '</div>',
                style: "width: 450px",
                onCancel: function() {
                    this.destroyRecursive();
                    deferred.reject();
                }
            });
            var actionBar = domConstruct.toDom("<div class='dijitDialogPaneActionBar'></div>");
            domConstruct.place(actionBar,messageDialog.containerNode);
            
            new Button({
                label: "Cancel",
                onClick: function() {
                    deferred.reject();
                    this.messageDialog.hide();
                    this.messageDialog.destroyRecursive();
                },
                messageDialog: messageDialog
            }).placeAt(actionBar);

            new Button({
                label: "Continue",
                onClick: function() {
                    deferred.resolve();
                    this.messageDialog.hide();
                    this.messageDialog.destroyRecursive();
                },
                messageDialog: messageDialog
            }).placeAt(actionBar);

            messageDialog.show();
            return deferred.promise;
        }
    })
});