define([
    "dojo/_base/declare",
    "thornberry/app/Component/_EditorComponent",
    "thornberry/app/Component/_TableFieldEditorMixin",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "dojo/text!./Templates/PolicyEditor.html",
    "dojo/_base/lang",
    "dojo/_base/array",
    "dojo/dom-style",
    "dojo/dom-attr",
    "dojo/dom-construct",
    "dojo/store/Memory",
    "dojo/promise/all",
    "thornberry/app/utils",
    "thornberry/serverRequest",
    "thornberry/app/Component/PatientInsurance/BillRateEditor",
    "dijit/DropDownMenu",
    "dijit/MenuItem",
    "dijit/form/DropDownButton",
    "dojo/when",
    "thornberry/object/TableManager",
    "dijit/form/Form",
    "dijit/form/FilteringSelect",
    "dijit/form/ValidationTextBox",
    "dijit/form/DateTextBox",
    "dijit/form/Textarea",
    "dijit/form/CheckBox",
    "dijit/form/RadioButton",
    "dijit/form/CurrencyTextBox",
    "dijit/form/NumberSpinner",
    "dijit/form/Button",
    "dojox/form/CheckedMultiSelect",
    "thornberry/app/Component/PatientInsurance/SequenceEditor",
    "thornberry/app/Component/InsuranceEligibility/PolicyLog"
],function(
    declare,
    _EditorComponent,
    _TableFieldEditorMixin,
    _TemplatedMixin,
    _WidgetsInTemplateMixin,
    template,
    lang,
    arrayUtils,
    domStyle,
    domAttr,
    domConstruct,
    Memory,
    all,
    utils,
    serverRequest,
    BillRateEditor,
    DropDownMenu,
    MenuItem,
    DropDownButton,
    when,
    TableManager
) {

    TableManager.add("Timing",new Memory({data:[
        { id: "Early", name: "Early" },
        { id: "Late", name: "Late" }
    ]}));

    return declare([_EditorComponent,_TemplatedMixin,_WidgetsInTemplateMixin,_TableFieldEditorMixin], {
        templateString: template,

        utils: utils,

        // internals
        lastTempBillRateEditorId: 0,
        lastEditorId: 0,
        billRateEditorStore: null,
        removedBillRateIds: null,

        getInitialProps: function() {
            return lang.mixin(this.inherited(arguments),{
                value: {
                    addSequence: 1,
                    billRates: [],
                    modifiedBillRates: [],
                    removedBillRateIds: []
                },
                mode: "edit",
                policies: [],
                sequences: [],
                patientType: null,
                ndocNum: null,
                priorBillRatePolicies: [],
                billingComponentSettings: {}
            });
        },

        constructor: function() {
            var t = this;
            t.billRateEditorStore = new Memory();
            t.removedBillRateIds = [];
        },

        buildRendering: function() {
            var t = this;
            t.inherited(arguments);

            TableManager.get("Tables.ChargeCodes",true);
            when(TableManager.get("FillInGroup~20624"),function(store) {
                var addAllMenu = new DropDownMenu();
                arrayUtils.forEach(store.data, function(chargeType) {
                    // Exclude System & SIA types
                    if (chargeType.id != 6 && chargeType.id != 7) {
                        var option = new MenuItem({
                            label: chargeType.name,
                            onClick: function() { t.addAllBillRates(chargeType.id); }
                        });
                        addAllMenu.addChild(option);
                    }
                });
                t.addAllBillRateButton = new DropDownButton({
                    dropDown: addAllMenu,
                    label: "Add ALL"
                },t.addAllNode);
            });

            t.copyFromButton.set("dropDown",new DropDownMenu());

            // validation
            t.addValidation("effectiveDate",utils.validators.dateBefore("stopDate","The effective date must be before the stop date."));
            t.addValidation("stopDate",utils.validators.dateAfter("effectiveDate","The stop date must be after the effective date."));
            t.addValidation("policyNum",function(name,value,lastValues) {
                var item = t.tableManager.get("Tables.Insurance").get(lastValues["insTabId"]);
                if (item && value) {
                    if (item.PolicyNumValidation == 1) {
                        return utils.validators.policyNum("medicare policy")(null,value);
                    } else if (item.PolicyNumValidation == 2) {
                        return utils.validators.policyNum("medicaid policy")(null,value);
                    }
                }
                return true;
            });
            
            // general logic
            t.addLogic("addSequence",function(name,value) {if (value.length === 0) {t.sequenceEditor.reset()}});
            t.addLogic("insTabId",function(name,value,lastValues) {
                domAttr.set(t.policyPrefixNode,"innerHTML","");
                domStyle.set(t.highRHCDaysContainer,"display","none");
                if (value) {
                    var item = t.tableManager.get("Tables.Insurance").get(value);
                    if (item) {
                        if (item.PolicyNumValidation == 1) {
                            domAttr.set(t.policyPrefixNode,"innerHTML",(t.props.patientType === "homecare" ? "[M0063]" : "[A0600B]"));
                        } else if (item.PolicyNumValidation == 2) {
                            domAttr.set(t.policyPrefixNode,"innerHTML",(t.props.patientType === "homecare" ? "[M0065]" : "[A0700]"));
                        }
                    }
                    serverRequest("Patient.Insurance.Utility","GetRHCHighAllowed",{
                        insuranceId: value,
                        ndocNum: t.props.ndocNum
                    }).then(function(allowed) {
                        if (allowed) {
                            domStyle.set(t.highRHCDaysContainer,"display","");
                        } else {
                            t.setFormValue("remainingHighRHCDays",null);
                        }
                    });
                }
                domStyle.set(t.billRateSectionContainer,"display",value && item && item.PatientNegotiatedRates == 1 ? "" : "none");
            },true);
            t.addLogic("insTabId",function(name,value,lastValues) {
                var policyNum = lastValues["policyNum"];
                var item = t.tableManager.get("Tables.Insurance").get(value);
                if (item && policyNum) {
                    var promise = null;
                    if (item.PolicyNumValidation == 1) {
                        promise = utils.validators.policyNum("medicare policy")(null,policyNum);
                    } else if (item.PolicyNumValidation == 2) {
                        promise = utils.validators.policyNum("medicaid policy")(null,policyNum);
                    }
                    if (promise) {
                        promise.then(function(valid) {
                            if (!valid) {
                                t.setFormValue("policyNum","",true);
                            }
                        });
                    }
                }
                if (item && item.PatientNegotiatedRates != 1) {
                    var editorIds = arrayUtils.map(t.billRateEditorStore.data,function(editorItem) { return editorItem.id; });
                    arrayUtils.forEach(editorIds,function(id) { t.removeBillRate(id); });
                }
            });
            t.addLogic(["guarRelationship"],function(name,value) {
                var item = t.tableManager.get("Tables.Relationship").get(value);
                if (item && item.isSelf == 1) {
                    serverRequest("UI.Utility","GetSelfData",t.props.ndocNum).then(function(result) {
                        t.setFormValue("guarName",result.name);
                        t.setFormValue("guarAddress1",result.address1);
                        t.setFormValue("guarAddress2",result.address2);
                        t.setFormValue("guarCity",result.city);
                        t.setFormValue("guarState",result.state);
                        t.setFormValue("guarZip",utils.conversions.zip("",result.zip,"set"));
                        t.setFormValue("guarSSN",utils.conversions.ssn("",result.ssn,"set"));
                        t.setFormValue("guarGender",result.sex);
                        t.setFormValue("guarPhone",utils.conversions.phone("",result.homePhone,"set"));
                        t.setFormValue("guarPhoneExt",result.homePhoneExt);
                    });
                }
            });
            t.addLogic(["guarZip"],function(name,value) {
                serverRequest("UI.Utility","GetCityAndState",{zipCode:value}).then(function(result) {
                    if (result.city && result.state) {
                        if (name === "guarZip") {
                            t.setFormValue("guarCity",result.city);
                            t.setFormValue("guarState",result.state);
                        }
                    }
                });
            });

            t.addLogic("allChargesUseGrossRate",function(name,value) {
                arrayUtils.forEach(t.billRateEditorStore.data,function(item) {
                    item.editor.set("props",{
                        value: item.editor.get("value"),
                        useGrossRate:!!value.length
                    });
                });
            },true);

            // disable logic
            t.addDisabled("insTabId",function(name,data) {return t.props.mode === "edit";});
            t.addDisabled("policySequence",function(name,data) {return data.addSequence.length === 0;});

            // requirement logic
            t.addRequirement([
                "insTabId","policyNum","effectiveDate"
            ],true);
            t.addRequirement(["relationTypeId"],function() {
                return t.props.billingComponentSettings && t.props.billingComponentSettings.enabledNow;
            });
            t.addRequirement("firstCertTiming",function() {
                // Require is this is a PPS insurance, we're adding a new insurance, and the billing component is enabled
                var selectedInsurance = t.getSelectedItem("insTabId");
                return t.props.mode == "add" && selectedInsurance && selectedInsurance.BillingType == 1 && t.props.billingComponentSettings && t.props.billingComponentSettings.enabledNow;;
            });

            // tables
            t.addTableFilter("insTabId",function(name,item,value) {
                return item.id == value || (item.active == "Yes" && (item.InsuranceType == "" || (item.InsuranceType == 1 && t.props.patientType == "homecare") || (item.InsuranceType == 2 && t.props.patientType == "hospice")));
            });
            t.addTableFilter(["relationTypeId","guarRelationship"],function(name,item,value) {
                return (item.active == "Yes" && item.hippa == true) || item.id == value;
            });

            t.addSubComponent(t.sequenceEditor);
        },

        refreshRendering: function() {
            var t = this;
            t.sequenceEditor.set("props",{ policies: t.props.policies });
            domStyle.set(t.policySequenceContainer,"display",(t.props.mode === "add" ? "" : "none"));

            t.addBillRateButton.set("disabled",t.props.disabled);
            t.copyAllBillRateButton.set("disabled",t.props.disabled);
            t.addAllBillRateButton && t.addAllBillRateButton.set("disabled",t.props.disabled);

            if (t.props.priorBillRatePolicies && t.props.priorBillRatePolicies.length > 0) {
                domStyle.set(t.copyFromButton.domNode,"display","");
                t.copyFromButton.dropDown.destroyDescendants();
                arrayUtils.forEach(t.props.priorBillRatePolicies,function(policy) {
                    t.copyFromButton.dropDown.addChild(new MenuItem({
                        label: policy.name,
                        onClick: function(){ t.copyBillRatesFrom(policy.id); }
                    }));
                });
                
            } else {
                domStyle.set(t.copyFromButton.domNode,"display","none");
            }
            t.copyFromButton.set("disabled",t.props.disabled);

            var eligibilityLog = t.props.value && t.props.value.eligibilityLog;
            if (eligibilityLog && eligibilityLog.length) {
                domStyle.set(t.eligibilityLogContainer,"display","");
                t.eligibilityLog.set("props",{
                    items: eligibilityLog
                });
            } else {
                domStyle.set(t.eligibilityLogContainer,"display","none");
            }

            t.inherited(arguments);
        },

        updateModified: function() {
            var t = this;
            var modified = t.removedBillRateIds.length || arrayUtils.some(t.billRateEditorStore.data,function(item) {
                return item.billRateId.toString().charAt(0) === "T" || item.editor.get("modified");
            });
            t.inherited(arguments,[modified]);
        },

        _addBillRate: function() {
            this.addBillRate();
        },

        addBillRate: function(data,init) {
            var t = this;
            data = data || null;
            var id = (data && data.id ? data.id : "T"+(++t.lastTempBillRateEditorId));

            var editor = BillRateEditor.getRecycled();
            var editorId = ++t.lastEditorId;
            t.billRateEditorStore.put({id: editorId, editor: editor, billRateId: id});

            editor.set("props",{ 
                value: data,
                disabled: t.props.disabled,
                useGrossRate: t.getField("allChargesUseGrossRate").get("checked"),
                onChangeModified: function() { if (!this.recycled) { 
                    t.invalidate("modified");} 
                },
                onCopy: function() { t.copyBillRate(editorId); },
                onRemove: function() { t.removeBillRate(editorId); }
            });
            
            // Hide the editor until it's finished initializing (but still put it in the container so the correct order is preserved)
            domStyle.set(editor.domNode,"display","none");
            t.billRateContainer.appendChild(editor.domNode);

            if (!init) {
                editor.readyPromise.then(function() {
                    domStyle.set(editor.domNode,"display","");
                });
                t.invalidate("modified");
            }
            return editor.readyPromise;
        },

        removeBillRate: function(id) {
            var t = this;
            var item = t.billRateEditorStore.get(id);
            if (item.billRateId.toString().charAt(0) !== "T") {
                t.removedBillRateIds.push(t.billRateEditorStore.get(id).billRateId);
            }
            t.billRateEditorStore.remove(id);
            item.editor.recycle();
            t.invalidate("modified");
        },
        
        copyBillRate: function(id) {
            var t = this;
            var item = t.billRateEditorStore.get(id);
            var data = item.editor.get("value");
            data.effectiveFromDate = data.effectiveToDate;
            data.effectiveToDate = "";
            t.addBillRate(data);
        },

        _copyAllActive: function() {
            var t = this;
            var chargeCodeIndex = {};
            var today = utils.conversions.date("",new Date(),"get");
            arrayUtils.forEach(t.billRateEditorStore.data,function(item) {
                var data = item.editor.get("value");
                if (!chargeCodeIndex[data.chargeCode] && (data.effectiveFromDate == "" || data.effectiveFromDate <= today) && (data.effectiveToDate == "" || data.effectiveToDate > today)) {
                    t.copyBillRate(item.id);
                    chargeCodeIndex[data.chargeCode] = true;
                }
            });
        },

        copyBillRatesFrom: function(id) {
            var t = this;
            serverRequest("Patient.Insurance.InsuranceService","GetBillRates",id).then(function(results) {
                arrayUtils.forEach(results,function(billRate) {
                    billRate.id = ""; // Clear the ID from the existing bill rate so that a temp ID will be generated instead
                    t.addBillRate(billRate);
                });
            },function() {
                utils.showMessage("An error was encountered.")
            });
        },

        addAllBillRates: function(chargeType) {
            var t = this;
            // Build index of already-used charge codes
            var selectedFieldValues = [];
            t.billRateEditorStore.query().forEach(function(item) {
                selectedFieldValues.push(item.editor.get("value").chargeCode);
            });

            var count = 0;
            TableManager.get("Tables.ChargeCodes").query({ ChargeType: chargeType }).forEach(function(chargeCode) {
                if (arrayUtils.indexOf(selectedFieldValues, chargeCode.id) == -1) {
                    count += 1;
                    if (count <= 25) {
                        
                        t.addBillRate({chargeCode: chargeCode.id});
                    } else if (count == 26) {
                        utils.showMessage("Only 25 charge codes in the selected category were added. Please use the option again to continue adding the rest.")
                    }
                }
            });
        },

        checkRequired: function() {
            return this.inherited(arguments) && arrayUtils.every(this.billRateEditorStore.data,function(item) {return item.editor.checkRequired()});
        },

        _getValueAttr: function() {
            var t = this;
            var value = t.inherited(arguments);

            // Set up bill rates from editors/removed IDs array
            var modifiedBillRates = [];
            arrayUtils.forEach(t.billRateEditorStore.data,function(item) {
                if (item.billRateId.toString().charAt(0) === "T" || item.editor.get("modified")) {
                    var value = item.editor.get("value");
                    value.id = item.billRateId;
                    modifiedBillRates.push(value);
                }
            });
            value.modifiedBillRates = modifiedBillRates;
            value.removedBillRateIds = t.removedBillRateIds.slice();
            return value;
        },

        _setDisabledAttr: function(value) {
            var t = this;
            if (value != t.disabled) {
                arrayUtils.forEach(t.billRateEditorStore.data,function(item) {
                    item.editor.set("props",{disabled:value});
                });
            }
            t.inherited(arguments);
        },

        _setValueAttr: function(value) {
            var t = this;
            var needsRefresh = t._needsRefresh(value);
            t.removedBillRateIds = [];

            t.inherited(arguments);
            t.value.modifiedBillRates = [];

            if (needsRefresh) {
                var existing = t.billRateEditorStore.data.length;
                var newCount = 0;
                if (value && value.billRates) {
                    var readyPromiseArray = [];

                    newCount = value.billRates.length;
                    for (var i = 0; i < newCount; i++) {
                        var billRate = value.billRates[i];
                        if (i < existing) {
                            var item = t.billRateEditorStore.data[i];
                            item.editor.set("props",{
                                value:billRate,
                                disabled:t.props.disabled
                            });
                            item.billRateId = billRate.id;
                        } else {
                            readyPromiseArray.push(t.addBillRate(billRate,true));
                        }
                    }

                    all(readyPromiseArray).then(function() {
                        arrayUtils.forEach(t.billRateEditorStore.data,function(item) {
                            domStyle.set(item.editor.domNode,"display","");
                        });
                    });
                }
                if (newCount < existing) {
                    for (var i = (existing - 1); i >= newCount; i--) {
                        var item = t.billRateEditorStore.data[i];
                        item.editor.recycle();
                        t.billRateEditorStore.remove(item.id);
                    }
                }
            }
        }
    });
});